from __future__ import annotations

from rq_interfaces.msg import Projection
from rq_interfaces.srv import AquireProjection, ReachabilityCheck

from rq_controller.common import PyProjection

import rclpy
from rclpy.node import Node

class HardwareClient(Node):

    def __init__(self):
        super().__init__('rq_hardware_client')
        self.cli_projection = self.create_client(AquireProjection, 'aquire_projection')
        while not self.cli_projection.wait_for_service(timeout_sec=1.0):
            self.get_logger().info('projection service not available, waiting again...')
        self.cli_reachability = self.create_client(ReachabilityCheck, 'reachability_check')
        while not self.cli_projection.wait_for_service(timeout_sec=1.0):
            self.get_logger().info('reachability service not available, waiting again...')
        
        self.req_projection = AquireProjection.Request()
        self.req_reachability = ReachabilityCheck.Request()

    def aquire_projection(self, projection: PyProjection) -> rclpy.Future:
        self.req_projection.scan_pose = projection.as_message()

        return self.cli_projection.call_async(self.req_projection)
    
    def check_reachability(self, projection: PyProjection) -> rclpy.Future:
        self.req_reachability.scan_pose = projection.as_message()

        return self.cli_projection.call_async(self.req_reachability)
    
    @staticmethod
    def projection_response_2_py(response: AquireProjection.Response) -> PyProjection:
        return PyProjection.from_message(response.projection)
    
    @staticmethod
    def reachability_response_2_py(response: ReachabilityCheck.Response) -> tuple[bool, float, PyProjection]:
        return (response.checked_scan_pose.reachable, 
                response.checked_scan_pose.cost, 
                PyProjection.from_message(response.checked_scan_pose.scan_pose))
    

def main():
    from matplotlib import pyplot as plt

    rclpy.init()

    minimal_client = HardwareClient()
    scan_pose = PyProjection.dummy()

    future = minimal_client.check_reachability(scan_pose)
    rclpy.spin_until_future_complete(minimal_client, future)

    response = future.result()
    reachability_check = minimal_client.reachability_response_2_py(response)
    minimal_client.get_logger().info(
        f'Pose is: {reachability_check[0]} \nMove cost: {reachability_check[1]}')

    future = minimal_client.aquire_projection(scan_pose)
    rclpy.spin_until_future_complete(minimal_client, future)

    response = future.result()
    projection = minimal_client.projection_response_2_py(response)
    minimal_client.get_logger().info(
        f'Received Projection ...')

    minimal_client.destroy_node()
    rclpy.shutdown()


    plt.imshow(projection.image)
    plt.show()

    


if __name__ == '__main__':
    main()